@@ -839,7 +839,8 @@ Handle
 Application_top_frame( Handle self, Handle from)
 {
 	while ( from) {
-		if ( kind_of( from, CWindow) && (
+		if (
+			kind_of( from, CWindow) && (
 			( PWidget( from)-> owner == application) ||
 			!CWidget( from)-> get_clipOwner(from)
 		))
