@@ -388,6 +388,20 @@ prima_kill_zombies( void)
 	}
 }
 
+void
+prima_refcnt_inc( Handle obj)
+{
+	if ( obj )
+		++SvREFCNT( SvRV((( PAnyObject) obj)-> mate));
+}
+
+void
+prima_refcnt_dec( Handle obj)
+{
+	if ( obj )
+		--SvREFCNT( SvRV((( PAnyObject) obj)-> mate));
+}
+
 void
 protect_object( Handle obj)
 {
@@ -880,6 +894,7 @@ call_perl_indirect( Handle self, char *subName, const char *format, Bool c_decl,
 		{
 		case 'i':
 		case 's':
+		case 'U':
 		case 'n':
 		case 'H':
 		case 'S':
@@ -919,6 +934,13 @@ call_perl_indirect( Handle self, char *subName, const char *format, Bool c_decl,
 				_string = va_arg( params, char *);
 				PUSHs( sv_2mortal( newSVpv( _string, 0)));
 				break;
+			case 'U':
+				_string = va_arg( params, char *);
+				_SV = newSVpv( _string, 0 );
+				_int = va_arg( params, int);
+				if ( _int ) SvUTF8_on(_SV);
+				PUSHs( sv_2mortal( _SV ));
+				break;
 			case 'n':
 				_number = va_arg( params, double);
 				PUSHs( sv_2mortal( newSVnv( _number)));
@@ -1258,6 +1280,8 @@ register_constants( void)
 	register_rgnop_constants();
 	register_fm_constants();
 	register_ggo_constants();
+	register_fv_constants();
+	register_dnd_constants();
 }
 
 XS( Object_alive_FROMPERL);
